<?php
namespace Teknisa\Repositories;

class Cronoprojeto extends \Doctrine\ORM\EntityRepository
{
    
    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $cdprojeto
     * @param $nrbaseline
     * @param $cdversao
     * @return mixed
     */
    public static function consultaProxSequencialCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline, $cdversao){
        $sql = "
            SELECT TO_CHAR(NVL(MAX(TO_NUMBER(CRP.CDSEQCRONPRJ)), 0)+1) AS CDSEQCRONPRJ
              FROM CRONOPROJETO CRP
             WHERE CRP.NRORG = :NRORG
               AND CRP.CDFILIAL = :CDFILIAL
               AND CRP.CDPROJETO = :CDPROJETO
               AND CRP.NRBASELINEPR = :NRBASELINEPR
               AND CRP.CDVERCRONPRJ = :CDVERCRONPRJ
        ";
        
        $params = array(
            'NRORG'        => $nrorg, 
            'CDFILIAL'     => $cdfilial, 
            'CDPROJETO'    => $cdprojeto, 
            'NRBASELINEPR' => $nrbaseline, 
            'CDVERCRONPRJ' => $cdversao         
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $cdprojeto
     * @param $nrbaseline
     * @param $cdversao
     * @return mixed
     */
    public static function consultaProxFilaCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline, $cdversao){
        $sql = "
            SELECT TO_NUMBER(LTRIM(MAX(CRP.CDFILAPRIO), '0'), 'XXX')+3 AS NRFILAPRIO,
                   LPAD(TRIM(TO_CHAR(TO_NUMBER(LTRIM(MAX(CRP.CDFILAPRIO), '0'), 'XXX')+3, 'XXX')), 3, '0') AS CDFILAPRIO
              FROM CRONOPROJETO CRP
             WHERE CRP.NRORG = :NRORG
               AND CRP.CDFILIAL = :CDFILIAL
               AND CRP.CDPROJETO = :CDPROJETO
               AND CRP.NRBASELINEPR = :NRBASELINEPR
               AND CRP.CDVERCRONPRJ = :CDVERCRONPRJ
        ";
        
        $params = array(
            'NRORG'        => $nrorg, 
            'CDFILIAL'     => $cdfilial, 
            'CDPROJETO'    => $cdprojeto, 
            'NRBASELINEPR' => $nrbaseline, 
            'CDVERCRONPRJ' => $cdversao         
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $cdprojeto
     * @param $nrbaseline
     * @return mixed
     */
    public static function consultaMaxVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline){
        $sql = "
            SELECT MAX(CDVERCRONPRJ) AS CDVERCRONPRJ
              FROM CRONOPROJETO CRP
             WHERE CRP.NRORG = :NRORG
               AND CRP.CDFILIAL = :CDFILIAL
               AND CRP.CDPROJETO = :CDPROJETO
               AND CRP.NRBASELINEPR = :NRBASELINEPR
        ";

        $params = array(
            'NRORG'        => $nrorg, 
            'CDFILIAL'     => $cdfilial, 
            'CDPROJETO'    => $cdprojeto, 
            'NRBASELINEPR' => $nrbaseline, 
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $cdprojeto
     * @param $nrbaseline
     * @param $cdversao
     * @return mixed
     */
    public static function consultaPercConclusaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline, $cdversao){
        $sql = "
            SELECT * 
              FROM (SELECT CRO.CDFILIAL, CRO.CDPROJETO, CRO.CDVERCRONPRJ, CRO.NRSOLISERV, CRO.NRTAREFA, CRO.NRPLATIVTARE,
                           NVL(CONCLUSAO_CRONO_INFO(CRO.CDFILIAL, CRO.CDPROJETO, CRO.CDVERCRONPRJ, CRO.NRSOLISERV, CRO.NRTAREFA, CRO.NRPLATIVTARE),0) AS VRPERCONCPRJ
                      FROM CRONOPROJETO CRO 
                     WHERE CRO.NRORG = :NRORG
                       AND CRO.CDFILIAL = :CDFILIAL
                       AND CRO.CDPROJETO = :CDPROJETO
                       AND CRO.NRBASELINEPR = :NRBASELINEPR
                       AND CRO.CDVERCRONPRJ = :CDVERCRONPRJ
                       AND CRO.CDNIVELITPRJ = '1')
             WHERE VRPERCONCPRJ > 0
        ";

        $params = array(
            'NRORG'        => $nrorg, 
            'CDFILIAL'     => $cdfilial, 
            'CDPROJETO'    => $cdprojeto, 
            'NRBASELINEPR' => $nrbaseline, 
            'CDVERCRONPRJ' => $cdversao, 
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $cdprojeto
     * @param $nrbaseline
     * @param $cdversao
     * @return mixed
     */
    public static function geraNovaVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline, $cdversao){
        $cronoprojeto = self::consultaMaxVersaoCronograma($nrorg, $cdfilial, $cdprojeto, $nrbaseline);        
        $cdmaxversao = $cronoprojeto[0]['CDVERCRONPRJ'];
        $cdnovaversao = \Geral\Util\Util::ZeroFill((string)((int)$cdmaxversao + 1), 3);
        
        $cdoperatual = \Geral\Util\Ambiente::getCdOperador();
        $dtatual = \Geral\Util\DateUtil::getDataAtual();
        $stdtatual = $dtatual->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        
        $sql = "
            INSERT INTO CRONOPROJETO (NRORG, CDFILIAL, CDPROJETO, NRBASELINEPR, CDVERCRONPRJ, CDSEQCRONPRJ, 
                                      VRPERCONCPRJ, VRPERCONCALC, NRSOLISERV, NRTAREFA, NRPLATIVTARE, 
                                      DSITCRONOPRJ, CDNIVELITPRJ, CDFILAPRIO, CDPRECITCRPJ, QTDIAFOLGACRON, 
                                      NRDIASITCRPJ, CDOPERADOR, IDMARCOPROJ, DTCRONOGPROJ, DTINIITCRPRJ, DTFINITCRPRJ, 
                                      DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, 
                                      DTULTATU, NRORGULTATU, CDOPERULTATU)
            SELECT NRORG, CDFILIAL, CDPROJETO, NRBASELINEPR, :CDNOVAVERSAO AS CDVERCRONPRJ, CDSEQCRONPRJ,
                   VRPERCONCPRJ, VRPERCONCPRJ, NRSOLISERV, NRTAREFA, NRPLATIVTARE, 
                   DSITCRONOPRJ, CDNIVELITPRJ, CDFILAPRIO, CDPRECITCRPJ, QTDIAFOLGACRON, 
                   NRDIASITCRPJ, CDOPERADOR, IDMARCOPROJ, DTCRONOGPROJ, DTINIITCRPRJ, DTFINITCRPRJ, 
                   TO_DATE(:DTATUAL, 'DD/MM/YYYY HH24:MI:SS') AS DTINCLUSAO, NRORGINCLUSAO, :CDOPERATUAL AS CDOPERINCLUSAO, 
                   TO_DATE(:DTATUAL, 'DD/MM/YYYY HH24:MI:SS') AS DTULTATU, NRORGULTATU, :CDOPERATUAL AS CDOPERULTATU
              FROM CRONOPROJETO
             WHERE NRORG        = :NRORG
               AND CDFILIAL     = :CDFILIAL
               AND CDPROJETO    = :CDPROJETO
               AND NRBASELINEPR = :NRBASELINEPR
               AND CDVERCRONPRJ = :CDVERCRONPRJ
        ";
        
        $params = array(
            'NRORG'        => $nrorg, 
            'CDFILIAL'     => $cdfilial, 
            'CDPROJETO'    => $cdprojeto, 
            'NRBASELINEPR' => $nrbaseline, 
            'CDVERCRONPRJ' => $cdversao, 
            'CDNOVAVERSAO' => $cdnovaversao, 
            'CDOPERATUAL'  => $cdoperatual, 
            'DTATUAL'      => $stdtatual, 
        );

        try{
            \Setforms2\Shared\Connection::getInstance()->execute($sql, $params);
            return $cdnovaversao;
        } catch(\Exception $e) {
            throw $e;
        }
    }

}